# HIVE AdKit C++ SDK

HIVE AdKit C++ SDK는 AD(X), ADOP등과 같이 Google Ad-Manager 제품을 C++ 인터페이스로
제공합니다.

HIVE AdKit C++ SDK는 HIVE SDK와 함께 사용하는 경우 RewardVideo시청 보상이 도달하였을때, HIVE Analytics에 시청 완료 정보를 전송합니다. HIVE SDK가 포함되지 않거나, HIVE SDK가 초기화가 되지 않은 경우에는 동작하지 않습니다.

HIVE AdKit C++ SDK는 *iOS*와 *Andriod*에서 동작하며, 각 플랫폼마다 필요한 설정요소의 차이가 있습니다.

## *iOS* 에서 사용

* iOS에서 사용하는 경우, Firebase-Framework와 포함된 HIVE AdKit SourceCode를 빌드하여 사용할 수 있습니다.

1. HIVEAdKit/src와 HIVEAdKit/include를 XCode프로젝트에 드레그&드랍의 동작으로 포함시킴니다.

   * .cpp .hpp .mm 파일을 모두 포함합니다.

   * HIVE AdKit 코드는 Brdige코드로 실제 사용성을 위해 제공합니다.

   * HIVE SDK v4의 CocosSample을 사용하고 있다면 HIVEAdKit/Sample_HIVE_SDK_v4를 함께 포함시킴니다.
1. XCode프로젝트의 BuildSetting탭에 Framework Search Path에 HIVEAdKit/firebase_cpp_sdk/frameworks/ios/universal 의 경로를 포함시킴니다.
1. XCode프로젝트의 BuildSetting탭에 Header Search Path에 HIVEAdKit/firebase_cpp_sdk/include 경로를 포함시킴니다.
1. XCode프로젝트의 BuildSetting탭에 Other Flags에 -framework "firebase" -framework "firebase_admob" 를 추가합니다.
1. CocoaPods의 설정으로 아래의 내용을 추가합니다.

    ``` pod
    pod 'Firebase/AdMob'
    ```

1. AD(X)를 사용하는 경우 CocoaPods의 설정으로 아래의 내용을 추가합니다.

    ``` pod
    source 'https://github.com/adxcorp/AdxLibrary_iOS_Release.git'
    source 'https://github.com/CocoaPods/Specs.git'

    pod 'ADXLibrary', '1.9.4'
    pod 'ADXLibrary-Vungle', '1.9.4'
    pod 'ADXLibrary-UnityAds', '1.9.4'
    pod 'ADXLibrary-Pangle', '1.9.4'
    pod 'ADXLibrary-Cauly', '1.9.4'
    pod 'ADXLibrary-FBAudienceNetwork', '1.9.4'
    ```

1. HIVE SDK v4 Sample을 사용하는 경우 다음의 내용을 추가합니다.

    'TopMenu.hpp'

    ``` h
    enum : int {
      ...
      BUTTON_ADMOB // 메뉴 버튼 추가
      MAX
    };
    ```

    'TopMenu.cpp'

    ``` cpp
    #include "AdMobTest.hpp" // 테스트 헤더 인크루드
    ...
    std::function<void(BaseView* view)> TopMenuChange[] = {
      ...
      MAKE_VIEW_CREATE(AdMobTest) // 테스트 UI 추가
    };
    const char* buttons[]={
      ...
      "AdMob" // 버튼 텍스트 추가
    };
    ```
    
1. iOS 14 이상을 대상으로 하는 경우 다음의 내용을 info.plist 파일에 추가합니다.

    * AD(X)의 SKAdNetwork 목록 : https://github.com/adxcorp/ADXLibrary_Integration/wiki/SKAdNetwork-ID-List

## *Android* 에서 사용

* Android Studio 3.0이상에서 빌드하는것을 추천합니다.
* Android에서 사용하는 경우, Prebuilt-Firebase-Library와 포함된 HIVE AdKit SourceCode를 빌드하여 사용할 수 있습니다.

1. ExternalNativeBuild

    * ndk-build를 사용하는 경우

    'build.gradle'

    ``` gradle
    externalNativeBuild {
        ndkBuild {
            targets 'MyGame'
            arguments '-j' + Math.max(Runtime.runtime.availableProcessors() - 1, 1)
            arguments 'NDK_TOOLCHAIN_VERSION=clang'
            // 일부 설정이 HOME을 인지하지 못하는 이슈가 있어 절대 경로를 사용한다.
            arguments 'COCOS_ROOT=' + System.getProperty('user.home') + '/cocos2d-x'
            arguments 'COCOS2DX=' + System.getProperty('user.home') + '/cocos2d-x'
            path 'jni/Android.mk'
        }
    }
    ndk {
            abiFilters 'x86', 'armeabi-v7a'//, 'arm64-v8a' // 'armeabi'는 더이상 지원하지 않음
        }
    ```

    'Android.mk'

    ``` mk
    $(call import-add-path,$(LOCAL_PATH)/../../..) #HIVEAdKit 설치 경로 모듈로 지정
    $(call import-module, HIVEAdKit) #HIVEAdKit 모듈설정 로드
    ...
    include $(BUILD_SHARED_LIBRARY) #NDK 빌드 타겟
    ...
    $(call import-module, HIVEAdKit/firebase_cpp_sdk) #HIVEAdKit 사용을 위한 firebase_cpp_sdk 모듈 prebuilt 로드
    ```

    * cmake를 사용하는 경우

    'build.gradle'

    ``` gradle
    externalNativeBuild {
        cmake {
            targets 'MyGame'
            arguments "-DCMAKE_FIND_ROOT_PATH=", "-DANDROID_STL=c++_static", "-DANDROID_TOOLCHAIN=clang", "-DANDROID_ARM_NEON=TRUE"
            cppFlags "-frtti -fexceptions -fsigned-char"
            path "jni/CMakeLists.txt"
        }
    }
    ndk {
            abiFilters 'x86', 'armeabi-v7a'//, 'arm64-v8a' // 'armeabi'는 더이상 지원하지 않음
        }
    ```

    'CMakeList.txt'

    ``` cmake
    set(HIVEADKIT_DIR_PATH ....../HIVEAdKit) #setup to HIVEAdKit path
    #HIVEAdKit library import after add_library( MyGame ... )
    add_subdirectory(${HIVEADKIT_DIR_PATH}) #HIVEAdKit library & macro load
    use_hiveadkit_prebuilt(${HIVEADKIT_DIR_PATH} MyGame) #library import automatic macro to build target
    ```

1. build.gradle 파일에 다음의 내용을 추가합니다.

    ``` gradle
    android {
      ...
      sourceSets.main {
          java.srcDirs "src" , "../../HIVEAdKit/Android/java" // HIVEAdKit JavaPlugin Source Path
      }
      ...
      dependencies {
        ...
        implementation "com.github.adxcorp.ADXLibrary_Android:adx-library-rewarded-video:1.17.1"
        implementation "com.github.adxcorp.ADXLibrary_Android:adx-library-standard:1.17.1"
      }
    }

    gradle.ext.firebase_cpp_sdk_dir = "../HIVEAdKit/firebase_cpp_sdk" // setup HIVEAdKit/firebase_cpp_sdk Path
    apply from: "$gradle.firebase_cpp_sdk_dir/Android/firebase_dependencies.gradle"
    firebaseCpp.dependencies {
      admob
    }
    ```

1. AdMob-AdManager Service를 사용하는 AD(X), ADOP를 사용하는 경우 AndroidManifest.xml 파일에 다음의 내용을 추가합니다.[1]

    ``` AndroidManifest.xml
    <application>
      <meta-data android:name="com.google.android.gms.ads.AD_MANAGER_APP" android:value="true" />
      <meta-data
        android:name="com.google.android.gms.ads.APPLICATION_ID"
        android:value="ca-app-pub-3940256099942544~3347511713"/> <!-- Sample value -->
    </application>
    ```

1. AD(X)를 사용하는 경우, build.gradle에 다음의 내용을 추가합니다.

    ``` gradle
    repositories {
        // ... other project repositories
        jcenter()
        maven { url 'https://jitpack.io' }
        maven { url 'https://maven.google.com' }
    }
    
    android {
        compileOptions {
           sourceCompatibility 1.8
           targetCompatibility 1.8
                }
        ...
        dependencies {
            ...
            // 리워드 비디오만 포함할 경우
            implementation 'com.github.adxcorp.ADXLibrary_Android:adx-library-rewarded-video:1.17.1'
            implementation "com.github.adxcorp.ADXLibrary_Android:adx-library-standard:1.17.1"
            // Google Play Service Ads 20.2.0 버전을 사용합니다.
            implementation 'com.google.android.gms:play-services-ads:20.2.0'
        }
    }
    ```

1. HIVE SDK v4 Sample을 사용하는 경우 다음의 내용을 추가합니다.

    'TopMenu.hpp'

    ``` h
    enum : int {
      ...
      BUTTON_ADMOB // 메뉴 버튼 추가
      MAX
    };
    ```

    'TopMenu.cpp'

    ``` cpp
    #include "AdMobTest.hpp" // 테스트 헤더 인크루드
    ...
    std::function<void(BaseView* view)> TopMenuChange[] = {
      ...
      MAKE_VIEW_CREATE(AdMobTest) // 테스트 UI 추가
    };
    const char* buttons[]={
      ...
      "AdMob" // 버튼 텍스트 추가
    };
    ```

1. Cocos2DX는 Flatbuffers 라이브러리를 사용하여 Firebase에 포함된 라이브러리와 중복이 발생합니다. 이경우, Flatbuffers 라이브러리를 빌드에 포함시키지 않거나, Firebase 오픈소스 라이브러리[C++ Quickstarts](https://github.com/firebase/quickstart-cpp)를 직접 빌드하여 사용합니다.[2]

    'cocos/prebuilt/android/Android.mk'

    ``` mk
    # LOCAL_WHOLE_STATIC_LIBRARIES += cocos_flatbuffers_static
    ```

    ``` cmake
    # cocos_flatbuffers_static # flatbuffers 라이브러리 제외
    ```

1. Firebase는 다음의 오픈소스 라이브러리를 포함하고 있습니다. 중복발생시 Firebase 오픈 소스 라이브러리를 통해 직접 빌드하는것을 권장합니다.

    |Library|
    |---|
    |CURL|
    |FLATBUFFERS|
    |LIBUV|
    |NANOPB|
    |UWEBSOCKETS|
    |ZLIB|

[1] https://developers.google.com/admob/android/quick-start#update_your_androidmanifestxml
[2] https://github.com/firebase/firebase-cpp-sdk
