//
//  AdMobTest.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by nanomech on 18/05/2020.
//
#include "HIVEAdKit.hpp"
#include "AdMobTest.hpp"

#if defined(ANDROID) // Android NDK predefined
#include <jni.h>
#include <HIVE_SDK_Plugin/HIVE_JniHelper.h>
// #include "HIVE_JniHelper.h"
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
#include "platform/android/jni/JniHelper.h"
#endif
#include "google_play_services/availability.h"
#endif


firebase::admob::AdParent getAdMobViewParent() {
    // TODO : getAdMobViewParent() for exclusive Cocos2D Engine.
    // TODO : getAdMobViewParent() implements for each Engine.
  #if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS) // Cocos2D predefined
  // Returns the iOS RootViewController's main view (i.e. the EAGLView).
    return (id)cocos2d::Director::getInstance()->getOpenGLView()->getEAGLView();
  #elif (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
  // Returns the Android Activity.
  return cocos2d::JniHelper::getActivity();
  #else
  // A void* for any other environments.
  return 0;
  #endif
}

#if defined(ANDROID)
JNIEnv* GetJniEnv() {
    // TODO : GetJniEnv() for exclusive Cocos2D Engine.
    // TODO : GetJniEnv() implements for each Engine.
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
    return cocos2d::JniHelper::getEnv();
#elif defined(ANDROID) // Android NDK predefined
    return hive::HiveJniHelper::getEnv();
#else
    return NULL;
#endif
}

jobject GetActivity() {
    // TODO : GetActivity() for exclusive Cocos2D Engine.
    // TODO : GetActivity() implements for each Engine.
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
    return cocos2d::JniHelper::getActivity();
#elif defined(ANDROID) // Android NDK predefined
//    return hive::HiveJniHelper::getActivity(); // Not Supported
    return NULL;
#else
    return NULL
#endif
}
#endif
using namespace hive;

bool AdMobTest::init()
{
    if (!BaseView::init())
        return false;

    setUseBackButton(true);
    setUseShowLogButton(true);

    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage) {
        addLogMessage(logMessage);
    });
    addLabel(":: HIVE Ad Manager Test ::");
    
    addDottedLine();
    addButton("INIT_ADMOB", BUTTON_INIT_ADMOB);
    addButton("INIT_REWARDED_VIDEO", BUTTON_INIT_REWARDED_VIDEO);
    addButton("LOAD_REWARDED_VIDEO", BUTTON_LOAD_REWARDED_VIDEO);
    addButton("SHOW_REWARDED_VIDEO", BUTTON_SHOW_REWARDED_VIDEO);
    addDottedLine();
    
    addButton("INIT_INTERSTITIAL", BUTTON_INIT_INTERSTITIAL_AD);
    addButton("LOAD_INTERSTITIAL", BUTTON_LOAD_INTERSTITIAL_AD);
    addButton("SHOW_INTERSTITIAL", BUTTON_SHOW_INTERSTITIAL_AD);
    addDottedLine();
    
    addToggleButton("BANNER POSITION", {"Top","Bottom","TopLeft","TopRight","BottomLeft","BottomRight"}, 0, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, BUTTON_ADAPTIVE_BANNER_POSITION);
    addButton("INIT_ADAPTIVE_BANNER", BUTTON_INIT_ADAPTIVE_BANNER);
    addButton("LOAD_ADAPTIVE_BANNER", BUTTON_LOAD_ADAPTIVE_BANNER);
    addButton("SHOW_ADAPTIVE_BANNER", BUTTON_SHOW_ADAPTIVE_BANNER);
    addButton("HIDE_ADAPTIVE_BANNER", BUTTON_HIDE_ADAPTIVE_BANNER);
    addDottedLine();

    
    setUIEnabled(false, BUTTON_INIT_REWARDED_VIDEO);
    setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
    setUIEnabled(false, BUTTON_SHOW_REWARDED_VIDEO);
    rewardVideo = NULL;
    
    setUIEnabled(false, BUTTON_INIT_INTERSTITIAL_AD);
    setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
    setUIEnabled(false, BUTTON_SHOW_INTERSTITIAL_AD);
    interstitial = NULL;
    
    setUIEnabled(false, BUTTON_INIT_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_LOAD_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_SHOW_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
    adaptiveBanner = NULL;
    bannerPosition = firebase::admob::BannerView::kPositionTop;

    return true;
}

void AdMobTest::onButtonEvent(int buttonTag)
{
    //    HIVEAdKit::RewardVideo::Initialize();
    //    //Load
    //    firebase::admob::AdRequest my_ad_request = {};
    //    HIVEAdKit::RewardVideo::Load("ca-app-pub-3940256099942544/1712485313", my_ad_request);
    //    //Show
    //    HIVEAdKit::RewardVideo::Show(getAdParent());
    
#if defined(ANDROID)
    const char* appId =                  "ca-app-pub-3940256099942544~3347511713";
    const char* rewardVideoUnitId =      "ca-app-pub-3940256099942544/5224354917";
    const char* InterstitialAdUnitId =   "ca-app-pub-3940256099942544/1033173712";
    const char* AdaptiveBannerAdUnitId = "ca-app-pub-3940256099942544/6300978111";
        
#else// UNITY_IOS
    const char* appId =                  "ca-app-pub-3940256099942544~1458002511";
    const char* rewardVideoUnitId =      "ca-app-pub-3940256099942544/1712485313";
    const char* InterstitialAdUnitId =   "ca-app-pub-3940256099942544/4411468910";
    const char* AdaptiveBannerAdUnitId = "ca-app-pub-3940256099942544/2934735716";
#endif
    
    /**
     BUTTON_INIT_ADMOB,
     BUTTON_INIT_REWARDED_VIDEO,
     BUTTON_LOAD_REWARDED_VIDEO,
     BUTTON_SHOW_REWARDED_VIDEO,
     BUTTON_INIT_INTERSTITIAL_AD,
     BUTTON_LOAD_INTERSTITIAL_AD,
     BUTTON_SHOW_INTERSTITIAL_AD,
     BUTTON_INIT_ADAPTIVE_BANNER,
     BUTTON_LOAD_ADAPTIVE_BANNER,
     BUTTON_SHOW_ADAPTIVE_BANNER,
     BUTTON_HIDE_ADAPTIVE_BANNER,
     */
    switch (buttonTag)
    {
        case BUTTON_INIT_ADMOB:{
            firebase::AppOptions appOptions = firebase::AppOptions();
            //App ID, API key, and Project ID must be specified in App options.
#ifdef __ANDROID__
            appOptions.set_app_id("331526026701");
            appOptions.set_api_key("AIzaSyAxOUz9NQtpZZD_XvfD5L7V1dZZ96tYwZ8");//
            appOptions.set_project_id("core-cocos-test");
#endif
            firebase::InitResult result = HIVEAdKit::AdMobInitialize(appOptions, appId);
            if(  result == firebase::kInitResultSuccess ) {
                setUIEnabled(false, BUTTON_INIT_ADMOB);
                setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
                setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
                setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
            } else if( result == firebase::kInitResultFailedMissingDependency ) {
#if defined(ANDROID) // Android NDK predefined
                google_play_services::Availability availabilityResult = google_play_services::CheckAvailability(GetJniEnv(),GetActivity());
                Logger::log("Error AdMobInitialize() => kInitResultFailedMissingDependency: %d", availabilityResult);
                if( availabilityResult == google_play_services::Availability::kAvailabilityAvailable ) {
                    google_play_services::MakeAvailable(GetJniEnv(), GetActivity());
                } else {
                    std::string errorMessage = "";
                    std::string errorTitle = "";
                    switch (availabilityResult) {
                        case google_play_services::Availability::kAvailabilityUnavailableDisabled:
                            errorTitle = "AvailabilityUnavailableDisabled";
                            errorMessage = "Google Play services is disabled in Settings.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailableInvalid:
                            errorTitle = "AvailabilityUnavailableInvalid";
                            errorMessage = "Google Play services is invalid.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailableMissing:
                            errorTitle = "AvailabilityUnavailableMissing";
                            errorMessage = "Google Play services is not installed.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailablePermissions:
                            errorTitle = "AvailabilityUnavailablePermissions";
                            errorMessage = "Google Play services does not have the correct permissions.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailableUpdateRequired:
                            errorTitle = "AvailabilityUnavailableUpdateRequired";
                            errorMessage = "Google Play services need to be updated.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailableUpdating:
                            errorTitle = "AvailabilityUnavailableUpdating";
                            errorMessage = "Google Play services is currently updating.";
                            break;
                        case google_play_services::Availability::kAvailabilityUnavailableOther:
                            errorTitle = "AvailabilityUnavailableOther";
                            errorMessage = "Some other error occurred.";
                            break;
                        default:
                            errorTitle = "Unknown";
                            errorMessage = "Unknown";
                    }
                    cocos2d::MessageBox(errorMessage.c_str(),errorTitle.c_str());
                }
#endif
            }
            break;
        }
            /// RewardedVideo
        case BUTTON_INIT_REWARDED_VIDEO:
            rewardVideo = HIVEAdKit::RewardVideo::Initialize(rewardVideoUnitId, this, getAdMobViewParent(),
                                               [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_INIT_REWARDED_VIDEO);
                    setUIEnabled(true, BUTTON_LOAD_REWARDED_VIDEO);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
            
        case BUTTON_LOAD_REWARDED_VIDEO:
        {
            //Load
            firebase::admob::AdRequest my_ad_request = {};
            HIVEAdKit::RewardVideo::Load(*rewardVideo,  my_ad_request, "CocosCPP-RewardVideo-Load-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
                    setUIEnabled(true, BUTTON_SHOW_REWARDED_VIDEO);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
        }
        case BUTTON_SHOW_REWARDED_VIDEO:
            //Show
            HIVEAdKit::RewardVideo::Show(*rewardVideo, "CocosCPP-RewardVideo-Show-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_SHOW_REWARDED_VIDEO);
                    setUIEnabled(true, BUTTON_LOAD_REWARDED_VIDEO);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
            /// Interstitial
        case BUTTON_INIT_INTERSTITIAL_AD:
            if( interstitial != nullptr ) HIVEAdKit::Interstitial::Destroy(*interstitial);
            interstitial = HIVEAdKit::Interstitial::Initialize(InterstitialAdUnitId, this, getAdMobViewParent(),
                                               [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_INIT_INTERSTITIAL_AD);
                    setUIEnabled(true, BUTTON_LOAD_INTERSTITIAL_AD);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
            
        case BUTTON_LOAD_INTERSTITIAL_AD:
        {
            //Load
            firebase::admob::AdRequest my_ad_request = {};
            HIVEAdKit::Interstitial::Load(*interstitial, my_ad_request, "CocosCPP-Interstitial-Load-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
                    setUIEnabled(true, BUTTON_SHOW_INTERSTITIAL_AD);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
        }
        case BUTTON_SHOW_INTERSTITIAL_AD:
            //Show
            HIVEAdKit::Interstitial::Show(*interstitial, "CocosCPP-Interstitial-Show-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_SHOW_INTERSTITIAL_AD);
                    setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
            /// Adaptive Banner
        case BUTTON_ADAPTIVE_BANNER_POSITION:
        {
            auto posIndex = getSelectedToggleButtonIndex(BUTTON_ADAPTIVE_BANNER_POSITION);
            bannerPosition = (firebase::admob::BannerView::Position)(posIndex);
            if( adaptiveBanner != NULL )
                HIVEAdKit::AdaptiveBanner::SetPosition(*adaptiveBanner, bannerPosition, [=](const firebase::Future< void >& completed_future) {
                    // We are probably in a different thread right now.
                    if (completed_future.error() == 0) {
    //                    DoSomethingWithResultData(completed_future.result());
                    }
                    else {
                        Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                    }
                });
            
            break;
        }
        case BUTTON_INIT_ADAPTIVE_BANNER:
            // Standard mobile banner size is 320x50.
            static const int kBannerWidth = 320;
            static const int kBannerHeight = 50;
            firebase::admob::AdSize banner_ad_size;
              banner_ad_size.ad_size_type = firebase::admob::kAdSizeStandard;
              banner_ad_size.width = kBannerWidth;
              banner_ad_size.height = kBannerHeight;
//            firebase::admob::BannerView::Position position = firebase::admob::BannerView::kPositionTop;
            adaptiveBanner = HIVEAdKit::AdaptiveBanner::Initialize(AdaptiveBannerAdUnitId,
                                                                   this,
                                                                   getAdMobViewParent(),
                                                                   banner_ad_size,
                                                                   bannerPosition,
                                               [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_INIT_ADAPTIVE_BANNER);
                    setUIEnabled(true, BUTTON_LOAD_ADAPTIVE_BANNER);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
            
        case BUTTON_LOAD_ADAPTIVE_BANNER:
        {
            //Load
            firebase::admob::AdRequest my_ad_request = {};
            HIVEAdKit::AdaptiveBanner::Load(*adaptiveBanner, my_ad_request, "CocosCPP-AdaptiveBanner-Load-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_LOAD_ADAPTIVE_BANNER);
                    setUIEnabled(true, BUTTON_SHOW_ADAPTIVE_BANNER);
                    setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
        }
        case BUTTON_SHOW_ADAPTIVE_BANNER:
            //Show
            HIVEAdKit::AdaptiveBanner::Show(*adaptiveBanner, "CocosCPP-AdaptiveBanner-Show-AdPlacementInfo",
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(false, BUTTON_SHOW_ADAPTIVE_BANNER);
                    setUIEnabled(true, BUTTON_HIDE_ADAPTIVE_BANNER);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
        case BUTTON_HIDE_ADAPTIVE_BANNER:
            //Hide
            HIVEAdKit::AdaptiveBanner::Hide(*adaptiveBanner,
                                             [=](const firebase::Future< void >& completed_future) {
                // We are probably in a different thread right now.
                if (completed_future.error() == 0) {
//                    DoSomethingWithResultData(completed_future.result());
                    setUIEnabled(true, BUTTON_SHOW_ADAPTIVE_BANNER);
                    setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
                }
                else {
                    Logger::log("Error %d: %s", completed_future.error(), completed_future.error_message());
                }
            });
            break;
    }
}

void AdMobTest::onAdLoaded() {
    Logger::log("onAdLoaded\n");
}
void AdMobTest::onAdOpening() {
    Logger::log("onAdOpening\n");
}
void AdMobTest::onAdClosed() {
    Logger::log("onAdClosed\n");
}
void AdMobTest::onUserEarnedReward(std::string type, double amount) {
    Logger::log("onUserEarnedReward\n");
}
void AdMobTest::onAdLeavingApplication() {
    Logger::log("onAdLeavingApplication\n");
}


//// Video Delegate
//void AdMobTest::onRewardedVideoAdOpened(){
//    Logger::log("onRewardedVideoAdOpened\n");
//}
//void AdMobTest::onRewardedVideoStarted(){
//    Logger::log("onRewardedVideoStarted\n");
//}
//void AdMobTest::onRewarded(){
//    Logger::log("onRewarded\n");
//}
//void AdMobTest::onRewardedVideoCompleted(){
//    Logger::log("onRewardedVideoCompleted\n");
//}
//void AdMobTest::onRewardedVideoAdClosed(){
//    Logger::log("onRewardedVideoAdClosed\n");
//}
