//
//  HIVEAdKit.hpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by nanomech on 06/04/2020.
//

#ifndef HIVEAdKit_hpp
#define HIVEAdKit_hpp

#include <stdio.h>
#include <string>
#include "firebase/admob.h"
#include "firebase/future.h"


#if TARGET_IPHONE_SIMULATOR
extern const char* _getSimulatorDevice(void);
#endif
extern "C" {
    void HIVEAnlayticsLog_AdLog(const char* eventType, const char* adUnitId, const char* uniqueSessionKey, const char* network, const char* adPlacementInfo, const char* rewardType, double rewardAmount);
    const char* HIVEAnalyticsLog_CreateUniqueSessionKey();
} // extern "C"

#if defined(__ANDROID__)
// Get the JNI environment.
JNIEnv* GetJniEnv();
// Get Android Activity
jobject GetActivity();
#endif  // defined(__ANDROID__)

extern firebase::admob::AdParent getAdMobViewParent();

class HIVEAdKit
{

private:
    const std::string TAG = "HIVEAdKit::";

public:
    static firebase::InitResult AdMobInitialize(const firebase::AppOptions& appOptions, const char* appId)
    {
#if defined(__ANDROID__)
        // Initialize Firebase for Android.
        firebase::App* app = firebase::App::Create(appOptions, firebase::kDefaultAppName, GetJniEnv(), GetActivity());
#else
        // Initialize Firebase for iOS.
        firebase::App* app = firebase::App::Create(appOptions);
#endif
        return firebase::admob::Initialize(*app, appId);
    }

public:
    class EventHandlers {
    public:
        virtual void onAdLoaded() = 0;
        virtual void onAdOpening() = 0;
        virtual void onAdClosed() = 0;
        virtual void onUserEarnedReward(std::string type, double amount) = 0;
        virtual void onAdLeavingApplication() = 0;
    };
    
private:
    
public:
    class AdLog
    {
    private:
//        static std::string adNetwork = "admob"; /// Fixed
        std::string adSessionKey;
        EventHandlers* callback;

    protected:
        std::string adUnitId;
        std::string adPlacementInfo;
        
    protected:
        AdLog(std::string unitId, EventHandlers* pEventHanders) {
            adUnitId = unitId;
            callback = pEventHanders;
            adSessionKey = std::string(HIVEAnalyticsLog_CreateUniqueSessionKey());
        }
        AdLog(const AdLog&) {
            
        }
        virtual ~ AdLog() {
            
        }
        
        // Common Event
        void AdLoaded(std::string adPlacementInfo) {
            // load
            HIVEAnlayticsLog_AdLog("load", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdLoaded();
        }
        
        void AdOpening() {
            // open
            HIVEAnlayticsLog_AdLog("open", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdOpening();
        }
        
        void AdClosed() {
            // closed
            HIVEAnlayticsLog_AdLog("closed", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdClosed();
        }
        
        // RewardedAd Event
        void UserEarnedRewareded(std::string rewardType, double rewardAmount) {
            // reward
            HIVEAnlayticsLog_AdLog("reward", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), rewardType.c_str(), rewardAmount);
            if( callback != NULL ) callback->onUserEarnedReward(rewardType, rewardAmount);
        }
        
        // InterstitialAd, AdaptiveBanner Event
        void AdLeavingApplication() {
            // click
            HIVEAnlayticsLog_AdLog("click", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdLeavingApplication();
        }
        
    };
    class Interstitial : public AdLog, firebase::admob::InterstitialAd::Listener
    {
    private:
        firebase::admob::InterstitialAd* interstitial_ad;
        
        Interstitial(std::string unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent, std::function<void (const firebase::Future<void> &)> result) : AdLog(unitId, pEventHanders) {
            interstitial_ad = new firebase::admob::InterstitialAd();
            interstitial_ad->Initialize(parent, unitId.c_str()).OnCompletion([=](const firebase::Future< void >& completed_future){
                interstitial_ad->SetListener(this);
                result(completed_future);
            });
        }
        ~Interstitial() {
            delete interstitial_ad;
        }
        
        void OnPresentationStateChanged(firebase::admob::InterstitialAd* ad,
                                        firebase::admob::InterstitialAd::PresentationState state) {
            switch (state) {
                case firebase::admob::InterstitialAd::kPresentationStateCoveringUI:
                    AdOpening();
                    break;
                case firebase::admob::InterstitialAd::kPresentationStateHidden:
                    AdClosed();
                    break;
                default:
                    break;
            }
        }
    public:
        Interstitial& operator=(const Interstitial &rhs) {
            return *this;  // 자기 자신을 참조를 반환합니다.
        }
        
        static Interstitial* Initialize(std::string unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent, std::function<void (const firebase::Future<void> &)> result)
        {
            return (new Interstitial(unitId, pEventHanders, parent, result));
        }
        static bool IsInitialize(Interstitial& ad) {
            return (ad.interstitial_ad->InitializeLastResult().status() == firebase::kFutureStatusComplete &&
                    ad.interstitial_ad->InitializeLastResult().error() == firebase::admob::kAdMobErrorNone);
        }
        static bool IsLoaded(Interstitial& ad) {
            return (ad.interstitial_ad->LoadAdLastResult().status() == firebase::kFutureStatusComplete &&
                    ad.interstitial_ad->LoadAdLastResult().error() == firebase::admob::kAdMobErrorNone);
        }
        static void Load(Interstitial& ad, firebase::admob::AdRequest request, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result) {
            if (IsInitialize(ad)) {
                // Register the device IDs associated with any devices that will be used to
                // test your app. Below are sample test device IDs used for making the ad request.
    #if TARGET_IPHONE_SIMULATOR
                static const char* kTestDeviceIDs[] =
                    {_getSimulatorDevice(),};
                request.test_device_id_count = sizeof(kTestDeviceIDs) / sizeof(kTestDeviceIDs[0]);
                request.test_device_ids = kTestDeviceIDs;
    #endif
                ad.interstitial_ad->LoadAd(request).OnCompletion([=, &ad](const firebase::Future< void >& completed_future){
                    if (completed_future.error() == 0) {
                        ad.AdLoaded(adPlacementInfo);
                    }
                    else {
                    }
                    result(completed_future);
                });
            }
        }
        static void Show(Interstitial& ad, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result) {
            if( IsLoaded(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                result(ad.interstitial_ad->Show());
            }
        }
        static void Destroy(Interstitial& ad) {
            if( IsInitialize(ad)) {
                delete &ad;
            }
        }
    };
    
    class AdaptiveBanner : public AdLog, firebase::admob::BannerView::Listener
    {
    private:
        firebase::admob::BannerView::PresentationState lastState;
        firebase::admob::BannerView* banner_view;
        AdaptiveBanner(const char* unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent, firebase::admob::AdSize size, firebase::admob::BannerView::Position position, std::function<void (const firebase::Future<void> &)> result) : AdLog(unitId, pEventHanders) {
            banner_view = new firebase::admob::BannerView();
            // 알수 없는 이유로 unitId는 const char* 타입으로 선언된것만 사용이 가능..
            banner_view->Initialize(parent, unitId, size).OnCompletion([=](const firebase::Future< void >& completed_future){
                banner_view->SetListener(this);
                banner_view->MoveTo(position);
                result(completed_future);
            });
            lastState = firebase::admob::BannerView::kPresentationStateHidden;
        }
        ~AdaptiveBanner() {
            banner_view->Destroy();
        }
        void OnPresentationStateChanged(firebase::admob::BannerView* ad,
                                        firebase::admob::BannerView::PresentationState state) {
            switch (state) {
                case firebase::admob::BannerView::kPresentationStateCoveringUI:
                    AdLeavingApplication();
                    break;
                case firebase::admob::BannerView::kPresentationStateVisibleWithoutAd:
                    break;
                case firebase::admob::BannerView::kPresentationStateVisibleWithAd:
                    if( lastState == firebase::admob::BannerView::kPresentationStateHidden ) AdOpening();
                    break;
                case firebase::admob::BannerView::kPresentationStateOpenedPartialOverlay:
                    break;
                case firebase::admob::BannerView::kPresentationStateHidden:
                    if( lastState == firebase::admob::BannerView::kPresentationStateVisibleWithAd ) AdClosed();
                    break;
                default:
                    break;
            }
            lastState = state;
            
        }
        void OnBoundingBoxChanged(firebase::admob::BannerView* ad,
                                  firebase::admob::BoundingBox box) {
            
        }
    public:
        AdaptiveBanner& operator=(const AdaptiveBanner &rhs) {
            return *this;  // 자기 자신을 참조를 반환합니다.
        }
        
        static AdaptiveBanner* Initialize(const char* unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent, firebase::admob::AdSize size, firebase::admob::BannerView::Position position, std::function<void (const firebase::Future<void> &)> result)
        {
            return (new AdaptiveBanner(unitId, pEventHanders, parent, size, position, result));
        }
        
        static bool IsInitialize(AdaptiveBanner& ad) {
            return (ad.banner_view->InitializeLastResult().status() == firebase::kFutureStatusComplete &&
                    ad.banner_view->InitializeLastResult().error() == firebase::admob::kAdMobErrorNone);
        }
        static bool IsLoaded(AdaptiveBanner& ad) {
            return (ad.banner_view->LoadAdLastResult().status() == firebase::kFutureStatusComplete &&
                    ad.banner_view->LoadAdLastResult().error() == firebase::admob::kAdMobErrorNone);
        }
        static void Load(AdaptiveBanner& ad, firebase::admob::AdRequest request, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result) {
            if (IsInitialize(ad)) {
                // Register the device IDs associated with any devices that will be used to
                // test your app. Below are sample test device IDs used for making the ad request.
    #if TARGET_IPHONE_SIMULATOR
                static const char* kTestDeviceIDs[] =
                    {_getSimulatorDevice(),};
                request.test_device_id_count = sizeof(kTestDeviceIDs) / sizeof(kTestDeviceIDs[0]);
                request.test_device_ids = kTestDeviceIDs;
    #endif
                ad.banner_view->LoadAd(request).OnCompletion([=, &ad](const firebase::Future< void >& completed_future){
                    if (completed_future.error() == 0) {
                        ad.AdLoaded(adPlacementInfo);
                    }
                    else {
                    }
                    result(completed_future);
                });
            }
        }
        static void Show(AdaptiveBanner& ad, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result) {
            if( IsLoaded(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                result(ad.banner_view->Show());
            }
        }
        static void Hide(AdaptiveBanner& ad, std::function<void (const firebase::Future<void> &)> result) {
            if( IsLoaded(ad)) {
                result(ad.banner_view->Hide());
            }
        }
        static void SetPosition(AdaptiveBanner& ad, firebase::admob::BannerView::Position position, std::function<void (const firebase::Future<void> &)> result) {
            if( IsInitialize(ad)) {
                result(ad.banner_view->MoveTo(position));
            }
        }
        static void Destory(AdaptiveBanner& ad, std::function<void (const firebase::Future<void> &)> result) {
            if( IsInitialize(ad)) {
                delete &ad;//result(ad.banner_view->Destroy());
            }
        }
    };
    
    class RewardVideo : public AdLog, firebase::admob::rewarded_video::Listener
    {
    private:
        inline static RewardVideo* pRV = 0;
    protected:
        firebase::admob::AdParent adParent;
        
        RewardVideo(std::string unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent) : AdLog(unitId, pEventHanders) {
//            firebase::admob::rewarded_video::SetListener(this);
            adParent = parent;
        }
        ~RewardVideo() {
//            firebase::admob::rewarded_video::Destroy();
        }
        void OnRewarded(firebase::admob::rewarded_video::RewardItem reward) {
            UserEarnedRewareded(reward.reward_type, reward.amount);
        }
        void OnPresentationStateChanged(firebase::admob::rewarded_video::PresentationState state) {
            switch (state) {
                case firebase::admob::rewarded_video::kPresentationStateCoveringUI:
                    AdOpening();
                    break;
                case firebase::admob::rewarded_video::kPresentationStateVideoHasStarted:
                    break;
                case firebase::admob::rewarded_video::kPresentationStateVideoHasCompleted:
                    break;
                case firebase::admob::rewarded_video::kPresentationStateHidden:
                    AdClosed();
                    break;
                default:
                    break;
            }
        }
    public:
        static RewardVideo* Initialize(std::string unitId, EventHandlers* pEventHanders, firebase::admob::AdParent parent, std::function<void (const firebase::Future<void> &)> result)
        {
            if( pRV != NULL )
                delete pRV;
            pRV = new RewardVideo(unitId, pEventHanders, parent);
            RewardVideo rv = *pRV;

            firebase::admob::rewarded_video::Initialize().OnCompletion([=,&rv](const firebase::Future< void >& completed_future){
                if (completed_future.error() == 0) {
                }
                else {
                }
                result(completed_future);
            });

            return pRV;
        }

        static bool IsInitialized() {
            if (firebase::admob::rewarded_video::InitializeLastResult().status() == firebase::kFutureStatusComplete &&
                firebase::admob::rewarded_video::InitializeLastResult().error() == firebase::admob::kAdMobErrorNone) {
                return true;
            }
            return false;
        }
        
        static void Load(RewardVideo& ad, firebase::admob::AdRequest& request, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result)
        {
            // Register the device IDs associated with any devices that will be used to
            // test your app. Below are sample test device IDs used for making the ad request.
#if TARGET_IPHONE_SIMULATOR
            static const char* kTestDeviceIDs[] =
                {_getSimulatorDevice(),};
            request.test_device_id_count = sizeof(kTestDeviceIDs) / sizeof(kTestDeviceIDs[0]);
            request.test_device_ids = kTestDeviceIDs;
#endif
            firebase::admob::rewarded_video::LoadAd(ad.adUnitId.c_str(), request).OnCompletion([=, &ad](const firebase::Future< void >& completed_future){
                if (completed_future.error() == 0) {
                    ad.AdLoaded(adPlacementInfo);
                }
                else {
                }
                result(completed_future);
            });;
        }
        
        static bool IsLoaded() {
            if (firebase::admob::rewarded_video::LoadAdLastResult().status() == firebase::kFutureStatusComplete &&
                firebase::admob::rewarded_video::LoadAdLastResult().error() == firebase::admob::kAdMobErrorNone) {
                return true;
            }
            return false;
        }

        // Show() 이후, RewardVideo 재 초기화 및 재 로드 필요
    public:
        static void Show(RewardVideo &ad, std::string adPlacementInfo, std::function<void (const firebase::Future<void> &)> result)
        {
            ad.adPlacementInfo = adPlacementInfo;
            firebase::admob::rewarded_video::SetListener(&ad);
            firebase::admob::rewarded_video::Show(ad.adParent).OnCompletion([=,&ad](const firebase::Future< void >& completed_future){
                if (completed_future.error() == 0) {
                }
                else {
                }
                result(completed_future);
            });
        }
    };
};

#endif /* HIVEAdKit_hpp */
